package com.mgh.pmdm.parques.db

import android.content.Context
import androidx.room.Database
import androidx.room.Room
import androidx.room.RoomDatabase
import androidx.room.migration.Migration
import androidx.sqlite.db.SupportSQLiteDatabase

// con la anotación @Database definimos una lista
// con las entidades con las que trabaja nuestra BD
// y su versión.
@Database(
    entities = [Park::class],
    version = 2
)
abstract class ParksDB : RoomDatabase() {
    // Métodos que devuelven los DAO que hemos implementado
    // Y con los cuales accederemos a la BD
    abstract fun parkDao(): ParkDAO
}

// Definimos la migración de la versión 1 a la 2
val MIGRATION_1_2 = object : Migration(1, 2) {
    override fun migrate(database: SupportSQLiteDatabase) {
        database.execSQL("ALTER TABLE Park ADD img VARCHAR")
    }
}


// Implementación del patrón Singleton
object DatabaseBuilder {

    private var INSTANCE: ParksDB? = null

    fun getInstance(context: Context): ParksDB {
        if (INSTANCE == null) {
            synchronized(ParksDB::class) {
                INSTANCE = buildRoomDB(context)
            }
        }
        return INSTANCE!!
    }

    // Constructor privado
    // Incorporamos la migración de la versión 1 a la 2
    private fun buildRoomDB(contexto: Context) =
        Room.databaseBuilder(
            contexto.applicationContext,
            ParksDB::class.java,
            "parks-db"
        ).addMigrations(MIGRATION_1_2).build()

}
